function [LLR,LE] = BCJRDecodingBlock_debug( Ly,gen_func,La,Lc,i_t)
%output LLR and Le given y, generator function, and EbNo(dB)



%%
%==================Initialize Variables===================================%
k=1;
[input,cs,ns,op]=RSCencoderTrellis(gen_func);%Trellis info for given
[n,K]=size(gen_func);%outputs from generator, constraint length
depth=length(Ly(1,:))/(n);%Length of trellis
no_of_branches=(2^(k*(K)));
no_of_nodes=(2^(k*(K-1)));
gamma=zeros(depth,no_of_branches);
gamma_p=gamma; %gamma values for the extrinsic LLR
alpha=zeros(depth+1,no_of_nodes);
beta=alpha;
alpha(1)=1;%initial value for all zero start
node_transitions=[bi2de(cs,'left-msb'),bi2de(ns,'left-msb')]+1;%state transitions


%%
%==================Pre-Decoding===================================%
%1. Split y into systematic and parity bits
ysyp=vec2mat(Ly(1,:),n);
%2. change input and op to soft inputs ie(1=>1 bit,-1=>0 bit)
input=2*input'-1;
op=2*op'-1;

%%
%===================Decoding Begins Here===============================%
%1. find gamma using equation exp(0.5*u_k*LA))*(exp(L_c*0.5*(x1y1+x2y2+...x_ny_n)))
% where u_k is the kth input bit
i=1;
while i<=size(ysyp,1)
    LA=repmat(La(i),no_of_branches,1)';
    y=repmat(real(ysyp(i,:)),no_of_branches,1)';
    aa=(0.5*input.*LA);
    bb=(Lc.*0.5.*sum((op.*y))) ;
    gamma_p(i,:)=exp(0.5.*Lc.*(op(2,:).*y(2,:)));
    gamma(i,:)=exp(aa+bb);%branch transition probability
    
    
    %2. find alpha
    left_pos=[];
    right_pos=[];
    pre_alpha=zeros(1,no_of_nodes);
    for j=1:no_of_nodes
        right_pos=(find(node_transitions(:,2)==j))'; %state s_i
        left_pos=(node_transitions(right_pos,1))'; %state s_{i-1}
        pre_alpha(j)=sum(alpha(i,left_pos).*gamma(i,right_pos));
    end
    alpha(i+1,:)=pre_alpha/sum(pre_alpha);%find \alpha_{i+1} normalized version
    i=i+1;
end
%%
%3. find beta
if sum(i_t)>0
    beta(end,1:end)=1/no_of_nodes;
else
    beta(end,1)=1;  % termination
end
k=depth;
while k>0
    left_pos=[];
    right_pos=[];
    pre_beta=zeros(1,no_of_nodes);
    for l=1:no_of_nodes
        left_pos=(find(node_transitions(:,1)==l))';
        right_pos=(node_transitions(left_pos',2))';
        pre_beta(l)=sum(gamma(k,left_pos).*beta(k+1,right_pos));
    end
    beta(k,:)=pre_beta/sum(pre_beta);
    k=k-1;
end
%%
%4. calculate LLR and Le
m=1;
while m<=depth
    alpha_beta=zeros(no_of_branches,1);
    for o=1:no_of_branches
        alpha_beta(o)=alpha(m,node_transitions(o,1)).*beta(m+1,node_transitions(o,2));
    end
    odd=1:2:no_of_branches;
    even=2:2:no_of_branches;
    
    top=sum(gamma(m,even).*alpha_beta(even)'); % as a result of the zeros
    top_gamma_p=sum(gamma_p(m,even).*alpha_beta(even)');
    
    bottom=sum(gamma(m,odd).*alpha_beta(odd)'); %as a result of the ones
    bottom_gamma_p=sum(gamma_p(m,odd).*alpha_beta(odd)');
    
    top_bottom=top+bottom;
    top_bottom_p=top_gamma_p+bottom_gamma_p;
    
    LLR(m)= (log(top)-log(top_bottom))-(log(bottom)-log(top_bottom));
    Le(m)=(log(top_gamma_p)-log(top_bottom_p))-(log(bottom_gamma_p)-log(top_bottom_p));
    LE(m)=(LLR(m)-La(m))-(Lc*ysyp(m,1));
    
    if  LLR(m)==inf | LLR(m)==-inf
        LLR(m)=100;
    end
    m=m+1;
end





end

